@@ -1,14 +1,34 @@
-### makes it easy to pass data to gnuplot - plotting tools - dumps data and generates one elementary plotting script
+"""
+This file is part of Autognuplotpy, autogpy.
+
+"""
 from __future__ import print_function
 
 import os
 import numpy as np
+import warnings
 from collections import OrderedDict
 
 from . import autognuplot_terms
 from . import plot_helpers
 
+try:
+    import pandas as pd
+    import pandas
+    pandas_support_enabled = True
+    # print("pandas support enabled")
+except:
+    pandas_support_enabled = False
 
+warnings.simplefilter('once', UserWarning)
+
+try:
+    import pygments
+    pygments_support_enabled = True
+except:
+    pygments_support_enabled = False
+
+    
 class AutoGnuplotFigure(object):
     """Creates an AutoGnuplotFigure object which wraps one gnuplot figure.
 
@@ -33,6 +53,8 @@ class AutoGnuplotFigure(object):
              (100) dpi of the jpg image showed in a jupyter notebook. It is used for the conversion of the pdf image produced by gnuplot.
         jpg_convert_quality: int, optional
              (100) conversion quality of the jpg image showed in a jupyter notebook. It is used for the conversion of the pdf image produced by gnuplot.
+        anonymous: bool, optional
+             (False) Specifies if a figure is generated in an anonymous folder. (Options as ssh sync and latex inclusion are turned off).
 
         Returns
         --------------------
@@ -42,9 +64,10 @@ class AutoGnuplotFigure(object):
         Examples
         ----------------
 
+        >>> # Usage case: no context manager, gnuplot syntax.
         >>> fig = AutoGnuplotFigure('folder','id')
         >>> # for some existing arrays x,y
-        >>> fig.p_generic('u 1 : 2 t "my title" ', x ,y) 
+        >>> fig.plot('u 1 : 2 t "my title" ', x ,y) 
         >>> fig.generate_gnuplot_file() 
         >>> # only jupyter
         >>> fig.jupyter_show_pdflatex(show_stdout=False) 
@@ -74,7 +97,8 @@ class AutoGnuplotFigure(object):
                  , allow_strings = False
                  , hostname = None
                  , jpg_convert_density = 100
-                 , jpg_convert_quality = 100):
+                 , jpg_convert_quality = 100
+                 , anonymous = False):
         """ Creates an AutoGnuplotFigure object
 
         :param folder_name: str
@@ -85,10 +109,10 @@ class AutoGnuplotFigure(object):
         :param tikz_enabled: Bool
         :param allow_strings: Bool
         :param hostname: str
+        :oaran anonymous: Bool
 
         """
         
-
         self.verbose = verbose
         
         self.folder_name = folder_name
@@ -98,7 +122,7 @@ class AutoGnuplotFigure(object):
         if not os.path.exists(self.folder_name):
             os.makedirs(self.folder_name)
             if self.verbose:
-                print( "created folder:", self.folder_name )
+                print("created folder:", self.folder_name)
 
         self.global_file_identifier = self.folder_name + '/' + self.file_identifier
         self.globalize_fname = lambda x : self.folder_name + '/' + x
@@ -170,6 +194,8 @@ class AutoGnuplotFigure(object):
             )
             )
 
+        self.is_anonymous = anonymous
+
     def set_figure_size(self,x_size=None, y_size=None, **kw):
         """Sets the terminal figure size and possibly more terminal parameters (string expected).
         """
@@ -181,6 +207,10 @@ class AutoGnuplotFigure(object):
 
         self.pdflatex_terminal_parameters.update(**kw)
 
+    def set_figure_linewidth(self,lw):
+        """Sets the global linewidth parameter for latex/tikz figures."""
+        self.pdflatex_terminal_parameters.update({'linewidth' : lw})
+
     def __wrap_text_section(self,content,wrapper):
 
         repl = dict(WRP=wrapper, CONTENT=content)
@@ -264,7 +294,101 @@ class AutoGnuplotFigure(object):
     def set_parameters(self,*args,**kw):
         """Proxies extend_global_plotting_parameters
         """
-        return self.extend_global_plotting_parameters(*args,**kw)
+        return self.alter_current_multiplot_parameters(*args,**kw)
+
+    def set(self,*args,**kw):
+        """expands to preamble param in each subplot, prepends the set command
+        """
+
+        add_set_statement = lambda x : "set " + x
+        if len(args):
+            args_with_set_in_front = [add_set_statement(str(x)) for x in args]
+            args_with_set_in_front_txt = "\n".join(args_with_set_in_front)
+        else:
+            args_with_set_in_front_txt = ""
+
+        ## never needs autoescaping. The args branches is easy.
+        self.set_parameters(args_with_set_in_front_txt)
+        
+        ## for some arguments autoescape is applied by self.__v_in_kw_needs_string, we should not apply it twice.
+        if len(kw):
+            kw_with_set_in_front = []
+            for k,v in kw.items():
+                k,v = self.__v_in_kw_needs_string(k,v)
+                self.set_parameters("set " + str(k) + " " + str(v), autoescape = False)
+                
+        #         kw_with_set_in_front.append("set " + str(k) + " " + str(v))
+        #     kw_with_set_in_front_txt = "\n".join(kw_with_set_in_front)            
+        # else:
+        #     kw_with_set_in_front_txt = ""
+        # self.set_parameters(args_with_set_in_front_txt,kw_with_set_in_front_txt)
+
+
+    def unset(self,*args,**kw):
+        """expands to preamble param in each subplot, prepends the unset command. For the kws the argument is ignored.
+        """
+
+        add_set_statement = lambda x : "unset " + x
+        if len(args):
+            args_with_set_in_front = [add_set_statement(str(x)) for x in args]
+            args_with_set_in_front_txt = "\n".join(args_with_set_in_front)
+        else:
+            args_with_set_in_front_txt = ""
+
+        if len(kw):
+            kw_with_set_in_front = []
+            for k,v in kw.items():
+                k,v = self.__v_in_kw_needs_string(k,v)
+                kw_with_set_in_front.append("unset " + str(k))
+
+            kw_with_set_in_front_txt = "\n".join(kw_with_set_in_front)
+            
+        else:
+            kw_with_set_in_front_txt = ""
+
+        self.set_parameters(args_with_set_in_front_txt,kw_with_set_in_front_txt)        
+
+        
+    def __v_in_kw_needs_string(self,k,v):
+        """determines wheter v needs to be wrapped in quotes ("<v>") or in dollars and quotes ("$<v>$").
+
+        prepending `s__` or appending `__s` to `k` yields stringification;  
+        prepending `s__` or appending `__s` to `k` yields stringification and wrapping in `$` signs.
+        """
+        if k.startswith('s__') \
+           or k.endswith('__s'):
+            # or k.startswith('str__') \
+            # or k.startswith('__str') \
+            # or k.startswith('S__') \
+            # or k.endswith('__S'):
+                        
+            needs_string = True
+            # maybe it was not a string, but we asked for a string.
+            v = str(v)
+            k = k.replace('s__','').replace('__s','')
+            
+        elif k.startswith('e__') \
+           or k.endswith('__e'):
+
+            needs_string = True
+            # maybe it was not a string, but we asked for a string.
+            v = "$" + str(v) + "$"  
+            k = k.replace('e__','').replace('__e','')
+            
+        else:
+            needs_string = False
+
+        if needs_string:
+            new_v = self.__autoescape_if_string(v
+                                                , add_quotes_if_necessary=True
+                                                # in this case we expect not to perform
+                                                # any string.format operation,
+                                                # thus we avoid to double curly brackets
+                                                , double_curly_brackets=False)
+            # print(new_v)
+            return k, new_v
+        else:
+            return k,str(v)
 
     def __enter__(self):
         return self
@@ -404,7 +528,6 @@ class AutoGnuplotFigure(object):
         >>> fig.p_generic('u 1 : 2 t "my title" ', x ,z)        
         """
         
-        
         autoescape = kw.get("autoescape", self._autoescape)
         escaped_args = []
         if autoescape:
@@ -626,22 +749,34 @@ class AutoGnuplotFigure(object):
         )        
         
         
-    def __autoescape_strings(self, command_line):
+    def __autoescape_strings(self, command_line, double_curly_brackets=True):
+        """autoescapes backslashes. Additionally, by default doubles the the curly brackets, to prevent them to disturb the a forthcoming string.format call.
+        """
         command_line = command_line.replace("\\","\\\\")
-        command_line = command_line.replace("{","{{")
-        command_line = command_line.replace("}","}}")
+        if double_curly_brackets:
+            command_line = command_line.replace("{","{{")
+            command_line = command_line.replace("}","}}")
 
         #preserves some needed blocks
         command_line = command_line.replace("{{DS_FNAME}}","{DS_FNAME}" )
         return command_line
 
+    def __quote_argument(self, v):
+        return '"%s"' % str(v)
+
     def __autoescape_if_string(self
                                , el
-                               , add_quotes_if_necessary = False):
+                               , add_quotes_if_necessary = False
+                               , double_curly_brackets = True):
         if isinstance(el,str):
-            escaped_str = self.__autoescape_strings(el)
-            Q = '"' if add_quotes_if_necessary else ""
-            return '{Q}{CONTENT}{Q}'.format(CONTENT=escaped_str,Q=Q)
+            escaped_str = self.__autoescape_strings(el
+                                                    , double_curly_brackets = double_curly_brackets)
+            if add_quotes_if_necessary:
+                return self.__quote_argument(escaped_str)
+            else:
+                return escaped_str
+            # Q = '"' if add_quotes_if_necessary else ""
+            # return '{Q}{CONTENT}{Q}'.format(CONTENT=escaped_str,Q=Q)
         else:
             return str(el)
 
@@ -661,25 +796,21 @@ class AutoGnuplotFigure(object):
 
         for k,v in kw.items():
             if not k in kw_reserved:
-                if k.startswith('s__') \
-                   or k.endswith('__s'):
-                    # or k.startswith('str__') \
-                    # or k.startswith('__str') \
-                    # or k.startswith('S__') \
-                    # or k.endswith('__S'):
-                        
-                    needs_string = True
-                    # maybe it was not a string, but we asked for a string.
-                    v = str(v)
-                    k = k.replace('s__','').replace('__s','')
-                else:
-                    needs_string = False
+                ## to replace with function __v_in_kw_needs_string
+                # if k.startswith('s__') \
+                #    or k.endswith('__s'):                        
+                #     needs_string = True
+                #     # maybe it was not a string, but we asked for a string.
+                #     v = str(v)
+                #     k = k.replace('s__','').replace('__s','')
+                # else:
+                #     needs_string = False
+                k,v = self.__v_in_kw_needs_string(k,v)
 
                 command_line = command_line + " " \
                     + k + " "\
-                    + self.__autoescape_if_string(v, add_quotes_if_necessary = needs_string)
-                    
-            
+                    + v #self.__autoescape_if_string(v, add_quotes_if_necessary = needs_string)
+                                
 
         ## needs to go after the blind adding to the command line,
         ## as this one only highjacks the title
@@ -722,7 +853,7 @@ class AutoGnuplotFigure(object):
                 
         Arguments
         ----------
-        command_line_or_data: string, list or np.array
+        command_line_or_data: string, list, `np.array` or `pd.Series`
              gnuplot command, without the explicit call to plot and the filename of the content.
              Alternatively, can be a list or np.array containing data (see *args)
         *args: lists or np.array, optional
@@ -735,7 +866,7 @@ class AutoGnuplotFigure(object):
              (False) set to True to allows columns with strings. This requires pandas. Might become True by default in the future.
         column_names: list of strings, optional
              (None) set the names of the columns. Considered only if `allow_strings=True`.
-        for_: string, optional
+        `for_`: string, optional
              (None) allows to use the `for` gnuplot keyword.
         label: string, optional
              (None) proxies the gnuplot `title` keyword.
@@ -750,16 +881,31 @@ class AutoGnuplotFigure(object):
         autoescape = kw.get("autoescape",self._autoescape)
         allow_strings = kw.get("allow_strings",self._allow_strings)
         column_names = kw.get("column_names",None)
-        for_enabled = kw.get("for_",None)
+        for_enabled = kw.get("for_",None)        
         if for_enabled is not None:
             allow_strings = False
             for_prepend = "for " + for_enabled
         else:
             for_prepend = ""
 
+
+        ## auto-wrapping the title with a string allowing to use t, ti, tit, titl, title kws.
+        # true if a title is provided for the plot, it any form
+        title_kw_provided = 'label' in kw
+
+        title_kw = 'title'
+        if not title_kw_provided:
+            for idx in range(len(title_kw)):
+                title_kw_attempt = kw.get(title_kw[0:idx+1],False)
+                if title_kw_attempt is not False:
+                    title_kw_provided = True
+                    kw['label'] = title_kw_attempt
+                    break
+
         ## the following keywords are not blindly appended to the command line
         kw_reserved = ["fname_specs", "autoescape", "allow_strings"
-                       , "column_names", "for_", "label"]
+                       , "column_names", "for_", "label"
+                       , "t", "ti", "tit", "titl", "title"]
 
 
         ### allowing to plot even without the command_line arg
@@ -770,17 +916,22 @@ class AutoGnuplotFigure(object):
             args = command_line,*args            
             command_line = ''
 
-        
+        # autosupport for pandas series
+        if pandas_support_enabled:
+            if len(args) == 1: #pd.core.series.Series
+                if isinstance(args[0],pandas.core.series.Series):
+                    series = args[0]
+                    args = series.index, series.values
+                    if not title_kw_provided:
+                        kw['label'] = str(series.name).replace("_"," ")
+                        title_kw_provided = True
+
 
         if autoescape:
             command_line = self.__autoescape_strings(command_line)
 
             if self.verbose:
                 print("autoescaping -- processing:", command_line)
-        
-        #prepend_parameters = kw.get("prepend_parameters",[""])
-        #fname_specs = ""
-
 
         if len(args) == 0: #case an explicit function is plotted:
             dataset_fname = None
@@ -807,7 +958,7 @@ class AutoGnuplotFigure(object):
 
             globalized_dataset_fname = self.globalize_fname(dataset_fname)
 
-            if allow_strings:
+            if allow_strings and pandas_support_enabled:
                 # pandas way. need to import
                 import pandas as pd
                 if column_names is None:
@@ -827,7 +978,6 @@ class AutoGnuplotFigure(object):
                     print(xyzt)
 
                 ##########
-
             else:
                 # numpy way
                 try:
@@ -841,7 +991,6 @@ class AutoGnuplotFigure(object):
                     print("\nWARNING: You got this exception likely beacuse you have columns with strings.\n"
                           "Please set 'allow_strings' to True.")
                     raise
-                
 
             if '"{DS_FNAME}"' not in command_line:
                 if self.verbose:
@@ -860,26 +1009,98 @@ class AutoGnuplotFigure(object):
                  #, 'prepended_parameters' : prepend_parameters
                 }
 
-
             self.__append_to_multiplot_current_dataset(
                 to_append
             )
             self.__dataset_counter += 1
 
         return to_append
+
+    def get_txt_dataset(self,ds_path):
+        """loads a txt dataset (proxies `np.loadtxt`)
+        """
+        ds = np.loadtxt(ds_path)
+        return ds
     
         
     def p_generic(self, command_line, *args, **kw):
-        """Proxies plot for backwards compatibility
+        """Proxies plot for backwards compatibility.
         """
         return self.plot(command_line,*args,**kw)
+
+    def fit(self,foo,modifiers,*args,**kw):
+
+        #modifiers = kw.get("modifiers","")
+        dataset_fname = self.file_identifier + self.datasetstring_template.format(            
+                DS_ID = self.__dataset_counter
+                , SPECS = "fit")
+
+        globalized_dataset_fname = self.globalize_fname(dataset_fname)
+
+        # this part needs to be refactored outputs
+        xyzt = list(map( lambda x : np.array(x), args  ))
+        # adding second dimension if needed, this is a feature to allow a for loop
+        xyzt = [  x[: , np.newaxis ] if len(x.shape) == 1 else x for x in xyzt]                   
         
+        data = np.concatenate( xyzt , axis = 1 )
+        np.savetxt( globalized_dataset_fname ,  data)
+        
+        
+        to_append = {"dataset_fname" : dataset_fname
+                     , "plottype" : "gnuplotfit"
+                     , "gnuplot_opt" : ""
+                     , 'gnuplot_command_template' : '{FOO} "{{DS_FNAME}}" {MODS}'.format(FOO = foo, MODS=modifiers) }
+        
+        self.__append_to_multiplot_current_dataset(
+                to_append
+        )
+        self.__dataset_counter += 1
+    
+
+    def fplot(self,foo,xsampling=None,
+              xsampling_N=100,
+              **kw):
+        """Mimicks matlab fplot function. 
+
+        Matlab ref: https://www.mathworks.com/help/matlab/ref/fplot.html
+
+        Parameters
+        ----------------
+        foo: scalar function
+            foo(n) must be evaluable.
+
+        xsampling: iterable, optional
+            (`np.linspace(-5,5)`) contains the x samples on which foo is evaluated.
+
+        **kw: same as in `plot`        
+        """
+
+        if xsampling is None:
+            xsampling = np.linspace(-5,5,xsampling_N)
+        elif isinstance(xsampling,tuple):
+            xsampling = np.linspace(xsampling[0],xsampling[1],xsampling_N)
+
+        yval = [foo(x) for x in xsampling]
+
+        return self.plot(xsampling,yval,**kw)
 
     
     def add_variable_declaration(self,name,value,is_string = False):
-        self.variables[name] = "'%s'"%str(value) if is_string else str(value)
+        """Add functions and variables declaration in the script preamble.
+
+        Parameters
+        ------------------------
+        name: string
+            Variable or function name. In case of a function needs to read like `"f(x)"`
+        value: value or string
+
+        is_string: bool, optional
+            (False) If `True` wraps value in double-quote signs. 
         
-        #self.global_plotting_parameters.append(  "{NAME}={VALUE}".format(NAME = name, VALUE = "'%s'"%str(value) if is_string else str(value) ) )
+        """
+        self.variables[name] = '"%s"'%str(value) if is_string else str(value)
+        
+
     def __render_variables(self):
 
         return "\n".join(
@@ -894,33 +1115,34 @@ class AutoGnuplotFigure(object):
                                         , self.datasets_to_plot):
             
             alterations_t = "\n".join( ["\n# this is multiplot idx: %d" % mp_count] + alterations + [""])
+
+            datasets_for_fit = filter( lambda x : x['plottype'] in ['gnuplotfit'], datasets )
+
+            
+            fit_call = "\n".join(
+                [ "fit " + x[ 'gnuplot_command_template' ].format( DS_FNAME= x[ 'dataset_fname'] )  for x in datasets_for_fit ] ) + "\n"
+            
+
+            datasets_for_plot = filter( lambda x : x['plottype'] in ['xyzt_gen', 'xy', 'expl_f']
+                                                  , datasets )
+
             plt_call = "p {ST}".format(ST = ",\\\n".join(
                 map( 
                     lambda x : x[ 'gnuplot_command_template' ].format( DS_FNAME = x[ 'dataset_fname' ] , OPTS =  x[ 'gnuplot_opt' ]  )
-                    , datasets
+                    , datasets_for_plot
                 )
             )
             )
-
-            calls.append(alterations_t + plt_call)
+            
+            calls.append(alterations_t + fit_call + plt_call)
             mp_count += 1
 
-        return "\n\n".join(calls)
+        return "\n".join(calls)
 
     def __generate_gnuplot_file_content(self):
         redended_variables = self.__render_variables()
-        parameters_string = "\n".join(self.global_plotting_parameters) + "\n\n"
-        
+        parameters_string = "\n".join(self.global_plotting_parameters) + "\n"        
 
-        
-
-        # plotting_string =  "p {ST}".format(ST = ",\\\n".join(
-        #     map(
-        #         lambda x : x[ 'gnuplot_command_template' ].format( DS_FNAME = x[ 'dataset_fname' ] , OPTS =  x[ 'gnuplot_opt' ]  )
-        #         , self.datasets_to_plot
-        #     )
-        # )
-        # )
 
         plotting_string = self.__generate_gnuplot_plotting_calls()
 
@@ -934,10 +1156,22 @@ class AutoGnuplotFigure(object):
         Parameters
         ----------------
         highlight: bool, optional
-             (False) Uses `pygaments` to color the gnuplot script
+             (True) Uses `pygments` to color the gnuplot script. 
+        linenos: string, optional
+             ("inline") Parameter `linenos` forwarded to the `pygments` `HtmlFormatter` object.  
         """
+        
         final_content = self.__generate_gnuplot_file_content()
-        if highlight:
+        try:
+            from IPython.display import display, HTML
+            get_ipython
+        except:
+            # jupyter seems not there..
+            highlight = False
+        
+        if highlight and pygments_support_enabled:
+            warnings.warn("This function benefits from pygments when installed.")
+            
             from pygments import highlight
             from pygments.lexers import GnuplotLexer
             from pygments.formatters import HtmlFormatter
@@ -946,7 +1180,11 @@ class AutoGnuplotFigure(object):
              
 
             from IPython.core.display import display, HTML
-            html__ =  highlight(final_content, GnuplotLexer(), HtmlFormatter(style='colorful', noclasses = False, linenos=linenos))
+            html__ =  highlight(final_content,
+                                GnuplotLexer(),
+                                HtmlFormatter(style='colorful',
+                                              noclasses = False,
+                                              linenos=linenos))
             display(HTML("""
             <style>
             {pygments_css}
@@ -955,16 +1193,17 @@ class AutoGnuplotFigure(object):
             
             display(HTML( html__ ) )
             #print(html__)
-        else:
-        
-            
+        else:            
             print (final_content)
 
-        
+    def get_gnuplot_file_content(self):
+        """returns the gnuplot core file content as a string
+        """
+        return self.__generate_gnuplot_file_content()
     
 
     def generate_gnuplot_file(self):
-        """Generates the final gnuplot scripts without creating any figure. Inclides Makefiles
+        """Generates the final gnuplot scripts without creating any figure. Includes: `Makefile`, `.gitignore` and synchronization scripts.
         """
 
         final_content = self.__generate_gnuplot_file_content()
@@ -1033,6 +1272,7 @@ class AutoGnuplotFigure(object):
                     , pdflatex_jpg_convert_quality = self.pdflatex_jpg_convert_quality
                 )
             )
+        ## the tikz part is refactored into a dedicated function
         self.__generate_gnuplot_files_tikz()
         
 
@@ -1090,16 +1330,27 @@ class AutoGnuplotFigure(object):
                       , stderr=_PIPE)
         output, err = proc.communicate()
 
+        #amending for the fit output
         was_there_an_error = \
             "error" in output or\
             "Error" in output or\
             "error" in err or\
-            "Error" in err or\
+            ("Error" in err and not "Standard Error" in err) or\
             proc.returncode != 0
         
         if was_there_an_error:
-            print("ERROR: an error was intercepted.")
-            print("     stderr and stdout reported below.")
+            
+            # diagnosing pdflatex installation problem
+            if "pdflatex: command not found" in err:
+                print("ERROR: PDFLATEX is NOT installed.\n")
+            elif "gnuplot: command not found" in err:
+                print("ERROR: GNUPLOT is NOT installed.\n")
+            else:
+                print("ERROR: an error was intercepted.")
+            
+            
+            print("  stderr and stdout reported below for diagnostics.")
+            print("")
         
         
         if self.verbose:
@@ -1113,14 +1364,23 @@ class AutoGnuplotFigure(object):
             print ("===== stdout =====")
             print (output)
             print ("=== stdout end ===")
-            
-        from IPython.core.display import Image, display
-        display(Image( image_to_display, height=height, width=width  ))
+
+        if not was_there_an_error:
+            from IPython.core.display import Image, display
+            display(Image( image_to_display, height=height, width=width  ))
 
     
 
     def jupyter_show(self                     
                      , show_stdout = False):
+        """Generates a figure via the jpg terminal and opens it in jupyter.
+        The more advanced `jupyter_show_pdflatex` and `jupyter_show_tikz` are advised. This call is left for debug.
+
+        Parameters
+        ----------------
+        show_stdout: bool, optional
+             (False) outputs `stdout` and `stderr` to screen.
+        """
         from subprocess import Popen as _Popen, PIPE as _PIPE, call as _call
 
         if self.verbose:
@@ -1177,14 +1437,12 @@ class AutoGnuplotFigure(object):
         solution: 
         in `/usr/share/gnuplot5/gnuplot/5.0/lua/gnuplot-tikz.lua`, replace:
 
-
         
         pgf.set_dashtype = function(dashtype)
         gp.write("\\gpsetdashtype{"..dashtype.."}\n")
         end
         
-
-
+        with
         
         pgf.set_dashtype = function(dashtype)
         gp.write("%\\gpsetdashtype{"..dashtype.."}\n")
@@ -1221,6 +1479,8 @@ class AutoGnuplotFigure(object):
         self.__scp_string_nofolder = "scp -r " + self.__ssh_string +"/*" + " ."
 
     def display_fixes(self):
+        """displays relevant fixes in case the `convert` call does not work or to solve a known gnuplot/luatex bug. 
+        """
         fixes_ =\
 """
 These are some fixes found for imagemagick and gnuplot tikz terminal
@@ -1254,6 +1514,9 @@ end
     def get_folder_info(self):
         from IPython.display import display, HTML
 
+        if self.is_anonymous:
+            raise Exception("get_folder_info disabled for anonymous figures.")
+
         infos = [
             ["(folder local):", self.folder_name ]
             , ["(folder global):", self.global_dir_whole_path]
@@ -1274,8 +1537,7 @@ end
                 )
             ))
 
-        except:
-        
+        except:        
             print ("(folder local): ", self.folder_name)
             print ("(folder global): ", self.global_dir_whole_path)
             print ("(ssh): " + self.__ssh_string  )
@@ -1292,6 +1554,58 @@ end
         """
         self.get_folder_info()
 
+    def print_latex_fig_inclusion_code(self):
+        return self.print_latex_snippet()
+
+    def print_latex_snippet(self):
+        """Propts latex code that can be used to include the figure.
+        
+        For the moment requires a call to self.generate_gnuplot_file()
+        """
+
+        if self.is_anonymous:
+            raise Exception("print_latex_snippet disabled for anonymous figures.")
+        
+        self.generate_gnuplot_file()
+
+        latex_incl_statement = plot_helpers.latex_document_include_figure_statement(
+            self.folder_name + '/' + self.local_pdflatex_output.replace(".pdf","") #strips out extension
+            , self.folder_name + '/' + self.__local_tikz_output.replace(".pdf","")
+            , tikz_enabled = self.terminals_enabled_by_default['tikz']['is_enabled']
+        )
+
+
+        try:
+            from IPython.display import display, HTML
+            get_ipython
+            highlight = True
+        except:
+            # jupyter seems not there..
+            highlight = False
+        
+        if pygments_support_enabled and highlight:            
+            from pygments import highlight
+            from pygments.lexers import TexLexer
+            from pygments.formatters import HtmlFormatter
+
+            from pygments.styles import get_style_by_name
+
+            html__ =  highlight(latex_incl_statement,
+                                TexLexer(),
+                                HtmlFormatter(style='colorful',
+                                              noclasses = False,
+                                              linenos=False))
+            display(HTML("""
+            <style>
+            {pygments_css}
+            </style>
+            """.format(pygments_css=HtmlFormatter().get_style_defs('.highlight'))))
+            
+            display(HTML( html__ ) )
+            
+        else:
+            print(latex_incl_statement)
+
     
             
                           
